from typing import Literal

COLORMAPS = Literal[
	'16_colors',
	'3-3-2 RGB',
	'3color-BMR',
	'3color-CGY',
	'3color-RMB',
	'3color-YGC',
	'5_ramps',
	'6_shades',
	'BOP blue',
	'BOP orange',
	'BOP purple',
	'Blue',
	'Circus Cherry',
	'Circus Cyan',
	'Circus Green',
	'Circus Ink Black',
	'Circus Ink Cherry',
	'Circus Ink Cyan',
	'Circus Ink Green',
	'Circus Ink Mint',
	'Circus Ink Purple',
	'Circus Ink Yellow',
	'Circus Mint',
	'Circus Purple',
	'Circus Yellow',
	'Cyan',
	'Cyan Hot',
	'Duo Cherry',
	'Duo Cyan',
	'Duo Green',
	'Duo Intense Cherry',
	'Duo Intense Cyan',
	'Duo Intense Green',
	'Duo Intense Mint',
	'Duo Intense Purple',
	'Duo Intense Yellow',
	'Duo Mint',
	'Duo Purple',
	'Duo Yellow',
	'Fire',
	'Grays',
	'Grays g=0.25',
	'Grays g=0.25 inverted',
	'Grays g=0.50',
	'Grays g=0.50 inverted',
	'Grays g=0.75',
	'Grays g=0.75 inverted',
	'Grays g=1.00 inverted',
	'Grays g=1.25',
	'Grays g=1.25 inverted',
	'Grays g=1.50',
	'Grays g=1.50 inverted',
	'Grays g=1.75',
	'Grays g=1.75 inverted',
	'Grays g=2.00',
	'Grays g=2.00 inverted',
	'Green',
	'Green Fire Blue',
	'HiLo',
	'I Blue',
	'I Bordeaux',
	'I Cyan',
	'I Forest',
	'I Green',
	'I Magenta',
	'I Purple',
	'I Red',
	'I Yellow',
	'ICA',
	'ICA2',
	'ICA3',
	'Ice',
	'Ink Black',
	'Ink Cherry',
	'Ink Cyan',
	'Ink Green',
	'Ink Mint',
	'Ink Purple',
	'Ink Wash Black',
	'Ink Wash Cherry',
	'Ink Wash Cyan',
	'Ink Wash Green',
	'Ink Wash Mint',
	'Ink Wash Purple',
	'Ink Wash Yellow',
	'Ink Yellow',
	'Magenta',
	'Magenta Hot',
	'OIMB1',
	'OIMB2',
	'OIMB3',
	'OPF fresh',
	'OPF orange',
	'OPF purple',
	'Orange Hot',
	'Parabolic RGB',
	'Phase Bold Green-Magenta',
	'Phase Bold Ink Green-Magenta',
	'Phase Bold Ink Mint-Cherry',
	'Phase Bold Ink Yellow-Cyan',
	'Phase Bold Mint-Cherry',
	'Phase Bold Yellow-Cyan',
	'Phase Green-Magenta',
	'Phase Ink Green-Magenta',
	'Phase Ink Mint-Cherry',
	'Phase Ink Yellow-Cyan',
	'Phase Mint-Cherry',
	'Phase Yellow-Cyan',
	'Pop blue',
	'Pop blue inverted',
	'Pop cherry',
	'Pop cherry inverted',
	'Pop cyan',
	'Pop cyan inverted',
	'Pop green',
	'Pop green inverted',
	'Pop lavender inverted',
	'Pop magenta',
	'Pop magenta inverted',
	'Pop mint',
	'Pop mint inverted',
	'Pop purple',
	'Pop purple inverted',
	'Pop red',
	'Pop red inverted',
	'Pop yellow',
	'Pop yellow inverted',
	'Quartetto MYGB Blue',
	'Quartetto MYGB Green',
	'Quartetto MYGB Magenta',
	'Quartetto MYGB Yellow',
	'Quartetto RYGB Blue',
	'Quartetto RYGB Green',
	'Quartetto RYGB Red',
	'Quartetto RYGB Yellow',
	'Rainbow RGB',
	'Red',
	'Red Hot',
	'Red/Green',
	'Spectrum',
	'Thermal',
	'Turbo',
	'Yellow',
	'Yellow Hot',
	'betterBlue',
	'betterCyan',
	'betterGreen',
	'betterOrange',
	'betterRed',
	'betterYellow',
	'blue_orange_icb',
	'brgbcmyw',
	'cool',
	'edges',
	'gem',
	'glasbey',
	'glasbey_inverted',
	'glasbey_on_dark',
	'glow',
	'mpl-inferno',
	'mpl-magma',
	'mpl-plasma',
	'mpl-viridis',
	'phase',
	'physics',
	'royal',
	'sepia',
	'smart',
	'thal',
	'thallium',
	'unionjack',
]
